import { Buff } from '@cmdcode/buff-utils';
import { SignedEvent } from './event.js';
import { Cipher } from './cipher.js';
import { isExpired, now } from './util.js';
const DEFAULTS = {
    cacheSize: 100,
    allowEcho: false,
    encryption: true,
    expiration: 1000 * 60 * 60 * 24,
    filter: { since: now() },
    inactiveLimit: 1000 * 60 * 60,
    kind: 21111,
    tags: []
};
export class NostrRoom {
    cache;
    client;
    config;
    events;
    _secret;
    connected;
    _sub;
    constructor(client, secret, config = {}) {
        this.cache = [];
        this.client = client;
        this.config = { ...DEFAULTS, ...config };
        this.connected = false;
        this.events = {};
        this._secret = Buff.str(secret);
        void this._subscribe();
    }
    get members() {
        const limit = this.config.inactiveLimit;
        const cache = (limit !== undefined)
            ? this.cache.filter(e => isExpired(e[2].created_at, limit))
            : this.cache;
        return cache.map(e => e[2].pubkey);
    }
    get roomId() {
        return this.shareKey.digest;
    }
    get shareKey() {
        return this._secret.digest;
    }
    async _subscribe() {
        const { filter, kind } = this.config;
        const { kinds = [] } = filter;
        const subFilter = {
            ...filter,
            kinds: [...kinds, kind],
            '#h': [this.roomId.hex]
        };
        this._sub = await this.client.sub([subFilter]);
        this._sub.on('event', (event) => {
            void this._eventHandler(event);
        });
        this._sub.on('eose', () => {
            this.connected = true;
        });
    }
    async _eventHandler(event) {
        const { allowEcho } = this.config;
        const pubkey = this.client.pubkey;
        const signed = new SignedEvent(event);
        const echoed = !allowEcho && signed.isAuthor(pubkey ?? '');
        if (echoed || signed.isExpired || !signed.isValid) {
            console.log(echoed, signed.isExpired, !signed.isValid);
            return;
        }
        let content = event.content;
        try {
            if (typeof content === 'string' && content.includes('?iv=')) {
                content = await Cipher.decrypt(content, this.shareKey);
            }
            const { eventName, payload } = JSON.parse(content);
            this.cache.push([eventName, payload, event]);
            if (this.cache.length > this.config.cacheSize) {
                this.cache.shift();
            }
            this.emit(eventName, payload, event);
        }
        catch (err) {
            this.emit('err', err);
        }
    }
    _getFn(eventName) {
        if (typeof this.events[eventName] === 'undefined') {
            this.events[eventName] = new Set();
        }
        return this.events[eventName];
    }
    emit(eventName, ...args) {
        const fns = [...this._getFn(eventName)];
        for (const fn of fns) {
            fn.apply(this, args);
        }
        const all = [...this._getFn('*')];
        for (const fn of all) {
            args = [eventName, ...args];
            fn.apply(this, args);
        }
    }
    async pub(eventName, payload, template = {}) {
        if (!this.connected) {
            throw new Error('Not connected to room!');
        }
        const { encryption, expiration, kind } = this.config;
        const { tags: conf_tags } = this.config;
        const { tags: temp_tags = [], ...rest } = template;
        try {
            let content = JSON.stringify({ eventName, payload });
            if (encryption) {
                content = await Cipher.encrypt(content, this.shareKey);
            }
            const tags = [
                ...conf_tags,
                ...temp_tags,
                ['h', this.roomId.hex],
                ['expiration', String(now() + expiration)]
            ];
            const envelope = { kind, ...rest, tags };
            const draft = { ...envelope, content };
            return await this.client.publish(draft);
        }
        catch (err) {
            console.error(err);
            return undefined;
        }
    }
    on(eventName, fn) {
        this._getFn(eventName).add(fn);
    }
    once(eventName, fn) {
        const onceFn = (...args) => {
            this.remove(eventName, onceFn);
            fn.apply(this, args);
        };
        this.on(eventName, onceFn);
    }
    within(eventName, fn, timeout) {
        const withinFn = (...args) => fn.apply(this, args);
        setTimeout(() => { this.remove(eventName, withinFn); }, timeout);
        this.on(eventName, withinFn);
    }
    remove(eventName, fn) {
        this._getFn(eventName).delete(fn);
    }
    prune(eventName) {
        this.events[eventName] = new Set();
    }
    leave() {
        this._sub?.unsub();
        this.connected = false;
    }
}
//# sourceMappingURL=room.js.map